﻿namespace TabsStudioSaver
{
    using TabExtensionList = System.Collections.Generic.List<string>;
    using TabList = System.Collections.Generic.List<
                        System.Collections.Generic.List<string>>;
    using TabsList = System.Collections.Generic.List<
                        System.Collections.Generic.List<
                            System.Collections.Generic.List<string>>>;

    class OrderController
    {
        public OrderController(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
        }

        public void Save()
        {
            System.Collections.Generic.List<TabsStudioExt.Tabs> allTabs = engine.ReturnTabs();
            if (allTabs.Count == 0)
                return;

            TabsList tabsList = new TabsList();            
            foreach(TabsStudioExt.Tabs tabs in allTabs)
            {
                System.Collections.Generic.List<TabsStudioExt.Tab> tabList = tabs.TabList;
                tabList.Sort(
                    delegate(TabsStudioExt.Tab lhs, TabsStudioExt.Tab rhs)
                    {
                        return tabs.TabPanel2.Children.IndexOf(lhs.TabItem).CompareTo(
                                    tabs.TabPanel2.Children.IndexOf(rhs.TabItem));
                    });
                TabList saverTabList = new TabList();
                foreach(TabsStudioExt.Tab tab in tabList)
                {
                    TabExtensionList tabExtensionList = new TabExtensionList();
                    foreach(TabsStudioExt.TabExtension tabExtension in tab.TabExtensions)
                    {
                        tabExtensionList.Add(GetWindowID(tabExtension.Window));
                    }
                    saverTabList.Add(tabExtensionList);
                }
                tabsList.Add(saverTabList);
            }
            try
            {
                using (System.IO.FileStream stream =
                    new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Create))
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(TabsList));
                    serializer.Serialize(stream, tabsList);
                }
            }
            catch (System.Exception)
            {
            }
        }

        public void Load()
        {
            tabsOrderList = null;
            try
            {
                using (System.IO.FileStream stream =
                    new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Open,
                        System.IO.FileAccess.Read, System.IO.FileShare.Read))
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(TabsList));
                    tabsOrderList = serializer.Deserialize(stream) as TabsList;
                }
            }
            catch (System.Exception)
            {
            }
        }

        public void OnTabCreated(TabsStudioExt.TabEventArgs e)
        {
            if (tabsOrderList == null)
                return;

            string tabCreatedID = GetWindowID(e.Tab.TabExtensions[0].Window);
            TabList tabList = FindTabList(tabCreatedID);
            if (tabList == null)
                return;

            foreach (System.Windows.Controls.TabItem tabItem in e.Tabs.TabPanel2.Children)
            {
                if (tabItem != e.Tab.TabItem)
                {
                    string id = GetWindowID(FindTabByItem(tabItem, e.Tabs).TabExtensions[0].Window);
                    if (LessID(tabList, tabCreatedID, id))
                    {
                        e.Tabs.TabPanel2.Children.Remove(e.Tab.TabItem);
                        e.Tabs.TabPanel2.Children.Insert(e.Tabs.TabPanel2.Children.IndexOf(tabItem), e.Tab.TabItem);
                        break;
                    }
                }
            }
        }

        private string GetStoragePath()
        {
            string solutionSettingsDir = GetSolutionSettingsDir();
            if (solutionSettingsDir != null)
            {
                return System.IO.Path.Combine(solutionSettingsDir, GetStringHash(dte.Solution.FullName.ToUpperInvariant()) + ".saver");
            }
            return System.IO.Path.ChangeExtension(dte.Solution.FullName, "tss");
        }

        private string GetSolutionSettingsDir()
        {
            try
            {
                return Microsoft.Win32.Registry.CurrentUser.OpenSubKey(@"Software\Sergey Vlasov\Tabs Studio").GetValue("SolutionSettingsDir") as string;
            }
            catch (System.Exception)
            {
            }
            return null;
        }

        private string GetStringHash(string s)
        {
            System.Security.Cryptography.SHA1CryptoServiceProvider sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
            return System.BitConverter.ToString(sha.ComputeHash(System.Text.Encoding.Unicode.GetBytes(s))).Replace("-", "");
        }

        private bool LessID(TabList tabList, string lhs, string rhs)
        {
            foreach (TabExtensionList tabExtensionList in tabList)
            {
                foreach (string s in tabExtensionList)
                {
                    if (string.Compare(s, lhs, true) == 0)
                        return true;
                    if (string.Compare(s, rhs, true) == 0)
                        return false;
                }
            }
            return false;
        }

        private TabsStudioExt.Tab FindTabByItem(System.Windows.Controls.TabItem tabItem, TabsStudioExt.Tabs tabs)
        {
            foreach(TabsStudioExt.Tab tab in tabs.TabList)
            {
                if (tab.TabItem == tabItem)
                    return tab;
            }
            return null;
        }

        private TabList FindTabList(string id)
        {
            foreach(TabList tabList in tabsOrderList)
            {
                foreach(TabExtensionList tabExtensionList in tabList)
                {
                    foreach(string s in tabExtensionList)
                    {
                        if (string.Compare(s, id, true) == 0)
                            return tabList;
                    }
                }
            }
            return null;
        }

        private string GetWindowID(EnvDTE.Window window)
        {
	        try	
            { 
                if (window.Document != null) // can throw
                    return window.Document.FullName;
                return window.Caption; // can throw
	        } 
            catch (System.Exception) 
            {
	        }
            return "";
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private TabsList tabsOrderList;
        EnvDTE80.DTE2 dte;
    }
}
