﻿namespace TabsStudioSaver
{
    class SolutionEventsSink : Microsoft.VisualStudio.Shell.Interop.IVsSolutionEvents
    {
        public SolutionEventsSink(OrderController orderController_)
        {
            orderController = orderController_;

            isSolutionOpening = false;

            timer = new System.Windows.Threading.DispatcherTimer();
            timer.Interval = System.TimeSpan.FromMilliseconds(500);
            timer.Tick += new System.EventHandler(TimerTick);
        }

        public bool IsSolutionOpening()
        {
            return isSolutionOpening;
        }

        public int OnAfterCloseSolution(System.Object pUnkReserved)
        {
            timer.Stop();
            isSolutionOpening = false;

            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterLoadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy,
            Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterOpenProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fAdded)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterOpenSolution(System.Object pUnkReserved, int fNewSolution)
        {
            isSolutionOpening = true;
            timerTicks = 0;
            timer.Start();

            try
            {
                orderController.Load();
            }
            catch (System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + System.Environment.NewLine + ex.StackTrace, 
                    "Saver (Unhandled exception)",
                    System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
            }
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnBeforeCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fRemoved)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnBeforeCloseSolution(System.Object pUnkReserved)
        {
            try
            {
                orderController.Save();
            }
            catch (System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + System.Environment.NewLine + ex.StackTrace,
                    "Saver (Unhandled exception)",
                    System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
            }
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnBeforeUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
            Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fRemoving, ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryCloseSolution(System.Object pUnkReserved, ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
            ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        private void TimerTick(object sender, System.EventArgs e)
        {
            ++timerTicks;
            if (timerTicks >= 5)
            {
                timer.Stop();
                isSolutionOpening = false;
            }
        }

        private OrderController orderController;
        private System.Windows.Threading.DispatcherTimer timer;
        private uint timerTicks;
        private bool isSolutionOpening;
    }
}
